import logging
import os

import pytest
import vcr

from aiotx.clients import AioTxBSCClient, AioTxETHClient

# ALL = "all"
# ANY = "any"
# NEW_EPISODES = "new_episodes"
# NONE = "none"
# ONCE = "once"

logging.getLogger("vcr").setLevel(logging.WARNING)
vcr_c = vcr.VCR(
    cassette_library_dir="tests/fixtures/cassettes",
    record_mode=os.environ.get("VCR_RECORD_MODE", "none"),
    match_on=["host", "path", "method", "query", "raw_body", "body"],
    filter_headers=["Authorization", "Cookie"],
    ignore_hosts=["127.0.0.1", "localhost"],
)

BSC_TEST_NODE_URL = "https://nameless-flashy-snow.bsc-testnet.quiknode.pro/c54e248a38fb9b7a8b31d84d57c1e41b203ed019/"
BSC_TEST_CHAIN_ID = 97

@pytest.fixture
def bsc_client() -> AioTxBSCClient:
    return AioTxBSCClient(BSC_TEST_NODE_URL, BSC_TEST_CHAIN_ID)


ETH_TEST_NODE_URL = "https://ethereum-sepolia-rpc.publicnode.com"
ETH_TEST_CHAIN_ID = 11155111

@pytest.fixture
def eth_client() -> AioTxETHClient:
    return AioTxETHClient(ETH_TEST_NODE_URL, ETH_TEST_CHAIN_ID)


class AioTxError(Exception):
    """
    Base exception for all aiotx errors.
    """
class BlockNotFoundError(AioTxError):
    pass

class InternalJSONRPCError(AioTxError):
    pass

class StackLimitReachedError(AioTxError):
    pass

class MethodHandlerCrashedError(AioTxError):
    pass

class ExecutionTimeoutError(AioTxError):
    pass

class NonceTooLowError(AioTxError):
    pass

class FilterNotFoundError(AioTxError):
    pass

class TraceRequestLimitExceededError(AioTxError):
    pass

class TransactionCostExceedsGasLimitError(AioTxError):
    pass

class NetworkError(AioTxError):
    pass

class VMExecutionError(AioTxError):
    pass

class MethodNotFoundError(AioTxError):
    pass

class InvalidRequestError(AioTxError):
    pass

class InvalidArgumentError(AioTxError):
    pass

class BlockRangeLimitExceededError(AioTxError):
    pass

class InternalJSONAioTxError(AioTxError):
    pass

class TransactionNotFound(AioTxError):
    pass

class ReplacementTransactionUnderpriced(AioTxError):
    pass

class WrongPrivateKey(AioTxError):
    pass


from aiotx.clients import AioTxBSCClient
import asyncio

client = AioTxBSCClient("NODE_URL", 97)
client2 = AioTxBSCClient("NODE_URL", 97)

@client.monitor.on_block
def handle_block(block):
    print("block", block)

@client.monitor.on_transaction
def handle_transaction(transaction):
    print("transaction", transaction)

@client2.monitor.on_block
def handle_block(block):
    print("client2: block", block)

@client2.monitor.on_transaction
def handle_transaction(transaction):
    print("client2: transaction", transaction)

async def main():
    monitoring_task1 = asyncio.create_task(client.start_monitoring(584))
    monitoring_task2 = asyncio.create_task(client2.start_monitoring(900))
    await asyncio.gather(monitoring_task1, monitoring_task2)

    try:
        while True:
            await asyncio.sleep(1)
    except KeyboardInterrupt:
        client.stop_monitoring()
        client2.stop_monitoring()


if __name__ == "__main__":
    asyncio.run(main())


from aiotx.clients._base_client import AioTxClient


class AioTxUTXOClient(AioTxClient):

    def __init__(self, node_url):
        self.node_url = node_url

    def generate_address(self):
        pass

    def get_balance(self, address):
        pass



from aiotx.clients import AioTxBSCClient
import asyncio

client = AioTxBSCClient("NODE_URL", 97)

@client.monitor.on_block
def handle_block(block):
    print("block", block)

@client.monitor.on_transaction
def handle_transaction(transaction):
    print("transaction", transaction)


async def main():
    await client.start_monitoring()

    try:
        while True:
            await asyncio.sleep(1)
    except KeyboardInterrupt:
        client.stop_monitoring()


if __name__ == "__main__":
    asyncio.run(main())


from enum import Enum


class BlockParam(Enum):
    LATEST = "latest"
    EARLIEST = "earliest"
    PENDING = "pending"
    SAFE = "safe"
    FINALIZED = "finalized"


class Wallet:
    def __init__(self) -> None:
        pass



import json

from ._bitcoin_base_client import AioTxUTXOClient
from ._evm_base_client import AioTxEVMClient


class AioTxBSCClient(AioTxEVMClient):
    def __init__(self, node_url, chain_id):
        super().__init__(node_url, chain_id)
        with open("aiotx/utils/bep20_abi.json") as file:
            bep_20_abi = json.loads(file.read())
        self._bep20_abi = bep_20_abi

    def _get_abi_entries(self):
        return [entry for entry in self._bep20_abi]


class AioTxETHClient(AioTxEVMClient):
    def __init__(self, node_url, chain_id):
        super().__init__(node_url, chain_id)
        with open("aiotx/utils/erc20_abi.json") as file:
            erc20_abi = json.loads(file.read())
        self._erc20_abi = erc20_abi

    def _get_abi_entries(self):
        return [entry for entry in self._erc20_abi]


class AioTxBTCClient(AioTxUTXOClient):
    pass


class AioTxLTCClient(AioTxUTXOClient):
    pass
